; ==================================================================================================
; GENERAL.INC - general routines for use throughout game.
; ==================================================================================================

Wait_2nd:
	bcall(_getcsc)
	cp sk2nd
	jr nz,Wait_2nd
	ret

; ==================================================================================================
; Pauses... until 2nd is pressed, whilst drawing funky greyscaled backer with "Paused" on top.
; ==================================================================================================

Pause:
	ld hl,PlotsScreen
	ld de,SavesScreen
	ld bc,768
	ldir

	ld d,%01010101
	ld hl,PlotsScreen
	call Mask
	
	ld d,%10101010
	ld hl,SavesScreen
	call Mask

	ld hl,Paused
	ld de,PlotsScreen+(12*25)
	ld bc,12*15
	call RLE

	ld hl,Paused
	ld de,SavesScreen+(12*25)
	ld bc,12*15
	call RLE
	
PauseLoop:
	call SwapBuffers
	call ionFastCopy

	ld a,KeyRow_Top
	out (1),a
	in a,(1)
	cp dk2nd
	ret z

	jr PauseLoop
	
; ==================================================================================================
; Swaps PlotsScreen and SavesScreen buffers.
; ==================================================================================================

SwapBuffers:
	ld ix,PlotsScreen
	ld de,SavesScreen
	ld bc,768
SwapLoop:
	ld a,(de)
	ld l,a
	ld a,(ix+0)
	ld (de),a
	ld (ix+0),l

	inc de
	inc ix
	dec bc

	ld a,b
	or c
	jr nz,SwapLoop
	ret

; ==================================================================================================
; Mask: Masks an area of data with "d", starting @ hl for 768 bytes.
; ==================================================================================================


Mask:	;In= d as mask.
	ld bc,768
MaskLoop:
	ld a,(hl)
	and d
	ld (hl),a
	inc hl
	dec bc
	ld a,c
	or b
	jr nz,MaskLoop
	ret

; ==================================================================================================
; RLE Decompress: hl (start of RLE data), bc (size of uncompressed data), de (Where to decomp to)
; ==================================================================================================

RLE:
_DispRLEL:
    ld a, (hl)          ; get the next byte
    cp $91              ; is it a run?
    jr z, _DispRLERun    ; then we need to decode the run
    ldi                 ; copy the byte, and update counters
_DispRLEC:
    ret po              ; ret if bc hit 0
    jr _DispRLEL         ; otherwise do next byte
_DispRLERun:
    inc hl
    inc hl              ; move to the run count
    ld a, (hl)          ; get the run count
_DispRLERunL:
    dec hl              ; go back to run value
    dec a               ; decrease run counter
    ldi                 ; copy byte, dec bc, inc de, inc hl
    jr nz, _DispRLERunL  ; if we're not done, then loop
    inc hl              ; advance the source pointer
    jr _DispRLEC         ; check to see if we should loop

; ==================================================================================================
; StartPause: Starts the pause counter.
; ==================================================================================================

StartPause:
	ld a,Pause_Length
	ld (PauseCounter),a
	ret

; ==================================================================================================
; CheckPause - Call in a loop. If pause is over, returns Z.
; ==================================================================================================

CheckPause:
	ld a,(PauseCounter)
	or a
	ret z
	dec a
	ld (PauseCounter),a
	cp 255
	ret

; ==================================================================================================
; Mine-shifting thingers.
; ==================================================================================================

Shift_Mine_L:	dec (ix+1) \ ret
Shift_Mine_R:	inc (ix+1) \ ret
Shift_Mine_U:	dec (ix+0) \ ret
Shift_Mine_D:	inc (ix+0) \ ret

; ==================================================================================================
; Greyscale Init: creates the two greyscale buffers.
; ==================================================================================================

Init_Grey:
	ld hl,PlotsScreen
	ld de,SavesScreen
	ld bc,768
	ldir

	ld hl,PlotsScreen
	ld d,%01010101
	call Mask

	ld hl,SavesScreen
	ld d,%10101010
	call Mask

	ret

; ==================================================================================================
; Increments ix by 4.
; ==================================================================================================
Add_ix_4:
	inc ix
	inc ix
	inc ix
	inc ix
	ret

; ==================================================================================================
; Inverts PlotsScreen
; ==================================================================================================

Screen_Invert:
	ld hl,PlotsScreen
	ld bc,768
Screen_Invert_Loop:
	ld a,(hl)
	cpl
	ld (hl),a
	inc hl
	dec bc
	ld a,c
	or b
	jr nz,Screen_Invert_Loop
	ret

; ==================================================================================================
; Funky routines to preset B to a variety of numbers :)
; ==================================================================================================

Set_b_40:	ld b,40 \ ret
Set_b_41:	ld b,41 \ ret
Set_b_42:	ld b,42 \ ret
Set_b_43:	ld b,43 \ ret


